/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;


/**
 * Unit tests for AntTaskInfo implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class AntTaskInfoTest extends TestCase {
    /**
     * The AntTaskInfo instance to test against.
     */
    private AntTaskInfo taskInfo = null;

    /**
     * The project to which the ant task belongs.
     */
    private Project project = null;

    /**
     * The name of the ant task used in logging messages.
     */
    private String taskName = null;

    /**
     * The location where the ant task was defined.
     */
    private Location location = null;

    /**
     * The container target of the ant task.
     */
    private Target target = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(AntTaskInfoTest.class);

        return suite;
    }

    /**
     * Prepares an AntTaskInfo instance for testing.
     */
    protected void setUp() {
        project = new Project();
        taskName = "taskName";
        location = new Location("build.xml");
        target = new Target();
        taskInfo = new AntTaskInfo(project, taskName, location, target);
    }

    /**
     * Tests the constructor with null project. Expects IllegalArgumentException.
     */
    public void testConstructor_NullProject() {
        try {
            new AntTaskInfo(null, taskName, location, target);
            fail("Creates AntTaskInfo with null project");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with null task name. Expects IllegalArgumentException.
     */
    public void testConstructor_NullTaskName() {
        try {
            new AntTaskInfo(project, null, location, target);
            fail("Creates AntTaskInfo with null task name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with empty task name. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyTaskName() {
        try {
            new AntTaskInfo(project, "", location, target);
            fail("Creates AntTaskInfo with empty task name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with null location. Expects IllegalArgumentException.
     */
    public void testConstructor_NullLocation() {
        try {
            new AntTaskInfo(project, taskName, null, target);
            fail("Creates AntTaskInfo with null location");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with null target. Expects IllegalArgumentException.
     */
    public void testConstructor_NullTarget() {
        try {
            new AntTaskInfo(project, taskName, location, null);
            fail("Creates AntTaskInfo with null target");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getProject method. Verifies if it returns the project set in constructor.
     */
    public void testGetProject() {
        assertEquals("Returns an incorrect project", project, taskInfo.getProject());
    }

    /**
     * Test of getTaskName method. Verifies if it returns the task name set in constructor.
     */
    public void testGetTaskName() {
        assertEquals("Returns an incorrect task name", taskName, taskInfo.getTaskName());
    }

    /**
     * Test of getLocation method. Verifies if it returns the location set in constructor.
     */
    public void testGetLocation() {
        assertEquals("Returns an incorrect location", location, taskInfo.getLocation());
    }

    /**
     * Test of getTarget method. Verifies if it returns the target set in constructor.
     */
    public void testGetTarget() {
        assertEquals("Returns an incorrect target", target, taskInfo.getTarget());
    }
}
