/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.tools.ant.BuildException;

import java.util.HashMap;
import java.util.Map;


/**
 * Unit tests for CredentialProperty implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class CredentialPropertyTest extends TestCase {
    /**
     * The CredentialProperty instance to test against.
     */
    private CredentialProperty credential = null;

    /**
     * The name of the credential property.
     */
    private String name = null;

    /**
     * The value of the credential property.
     */
    private String value = null;

    /**
     * The property map to add the credential property into.
     */
    private Map propertyMap = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(CredentialPropertyTest.class);

        return suite;
    }

    /**
     * Prepares a CredentialProperty instance for testing.
     */
    protected void setUp() {
        name = "name";
        value = "value";
        propertyMap = new HashMap();
        credential = new CredentialProperty(name, propertyMap);
    }

    /**
     * Tests the constructor with null name. Expects IllegalArgumentException.
     */
    public void testConstructor_NullName() {
        try {
            new CredentialProperty(null, propertyMap);
            fail("Creates CredentialProperty with null name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with empty name. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyName() {
        try {
            new CredentialProperty("", propertyMap);
            fail("Creates CredentialProperty with empty name");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with null property map. Expects IllegalArgumentException.
     */
    public void testConstructor_NullPropertyMap() {
        try {
            new CredentialProperty(name, null);
            fail("Creates CredentialProperty with null property map");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of setValue method with non-existing name. Verifies if the property name and value are added to the
     * property map.
     */
    public void testSetValue_NonExistName() {
        credential.setValue(value);
        assertEquals("Fails to add credential to property map", value, propertyMap.get(name));
    }

    /**
     * Test of setValue method with existing name. Expects BuildException.
     */
    public void testSetValue_ExistName() {
        // make sure the credential exists
        credential.setValue(value);

        try {
            credential.setValue(value);
            fail("Adds duplicate credential to property map");
        } catch (BuildException e) {
            // good
        }
    }
}
