/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;

import java.io.File;


/**
 * Unit tests for Server implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ServerTest extends TestCase {
    /**
     * The Server instance to test against.
     */
    private Server server = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(ServerTest.class);

        return suite;
    }

    /**
     * Prepares a Server instance for testing.
     */
    protected void setUp() {
        server = new SimpleServer("type");
    }

    /**
     * Tests the constructor with null type. Expects IllegalArgumentException.
     */
    public void testConstructor_NullType() {
        try {
            new SimpleServer(null);
            fail("Creates Server with null type");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with empty type. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyType() {
        try {
            new SimpleServer("");
            fail("Creates Server with empty type");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getDir and setDir methods with dir specified. Verifies it by getting the dir back after setting it.
     */
    public void testGetSetDir_Specified() {
        File dir = new File("");

        server.setDir(dir);
        assertEquals("Returns an incorrect dir", dir, server.getDir());
    }

    /**
     * Test of getDir method with no dir specified. Should return null.
     */
    public void testGetDir_Unspecified() {
        assertNull("Returns an incorrect dir", server.getDir());
    }

    /**
     * Test of setPort method with non-positive port. Expects IllegalArgumentException.
     */
    public void testSetPort_NonPositivePort() {
        // try zero port
        try {
            server.setPort(0);
            fail("Sets zero port");
        } catch (IllegalArgumentException e) {
            // good
        }

        // try negative port
        try {
            server.setPort(-1);
            fail("Sets negative port");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getPort and setPort methods with port specified. Verifies it by getting the port back after setting it.
     */
    public void testGetSetPort_Specified() {
        int port = 8080;

        server.setPort(port);
        assertEquals("Returns an incorrect port", port, server.getPort());
    }

    /**
     * Test of getPort method with no port specified. Should return -1.
     */
    public void testGetPort_Unspecified() {
        assertEquals("Returns an incorrect port", -1, server.getPort());
    }

    /**
     * Test of setServer method with empty server. Expects IllegalArgumentException.
     */
    public void testSetServer_EmptyServer() {
        try {
            server.setServer("");
            fail("Sets empty server");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getServer and setServer methods with server specified. Verifies it by getting the server back after
     * setting it.
     */
    public void testGetSetServer_Specified() {
        String name = "10.20.30.40";

        server.setServer(name);
        assertEquals("Returns an incorrect server", name, server.getServer());
    }

    /**
     * Test of getServer method with no server specified. Should return "localhost".
     */
    public void testGetServer_Unspecified() {
        assertEquals("Returns an incorrect server", "localhost", server.getServer());
    }

    /**
     * Test of getStartServer and setStartServer method with startserver specified. Verifies it by getting the
     * startserver back after setting it.
     */
    public void testGetSetStartServer_Specified() {
        // try true startserver
        server.setStartServer(true);
        assertTrue("Returns an incorrect startserver", server.getStartServer());

        // try false startserver
        server.setStartServer(false);
        assertFalse("Returns an incorrect startserver", server.getStartServer());
    }

    /**
     * Test of getStartServer method with no startserver specified. Should return true.
     */
    public void testGetStartServer_Unspecified() {
        assertTrue("Returns an incorrect startserver", server.getStartServer());
    }

    /**
     * Test of setAntTaskInfo method with null task info. Expects IllegalArgumentException.
     */
    public void testSetAntTaskInfo_NullTaskInfo() {
        try {
            server.setAntTaskInfo(null);
            fail("Sets null task info to server");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getAntTaskInfo and setAntTaskInfo methods with non-null task info. Verifies it by getting the task info
     * back after setting it.
     */
    public void testGetSetAntTaskInfo_NonNullTaskInfo() {
        AntTaskInfo taskInfo = new AntTaskInfo(new Project(), "taskName", new Location("build.xml"), new Target());

        server.setAntTaskInfo(taskInfo);
        assertEquals("Returns an incorrect task info", taskInfo, server.getAntTaskInfo());
    }

    /**
     * Test of getInitServer and setInitServer method with initserver specified. Verifies it by getting the initserver
     * back after setting it.
     */
    public void testGetSetInitServer_Specified() {
        // try true initserver
        server.setInitServer(true);
        assertTrue("Returns an incorrect initserver", server.getInitServer());

        // try false initserver
        server.setInitServer(false);
        assertFalse("Returns an incorrect initserver", server.getInitServer());
    }

    /**
     * Test of getInitServer method with no initserver specified. Should return true.
     */
    public void testGetInitServer_Unspecified() {
        assertTrue("Returns an incorrect initserver", server.getInitServer());
    }

    /**
     * Test of createCredentials method with non-existing credentials. Should return a CredentialsElement with empty
     * property map.
     */
    public void testCreateCredentials_NonExistCredentials() {
        CredentialsElement credentials = server.createCredentials();

        assertTrue("Returns an incorrect credentials element", credentials.getPropertyMap().isEmpty());
    }

    /**
     * Test of createCredentials method with existing credentials. Expects BuildException.
     */
    public void testCreateCredentials_ExistCredentials() {
        // make sure the credentials exists
        server.createCredentials();

        try {
            server.createCredentials();
            fail("Adds more than one credentials element");
        } catch (BuildException e) {
            // good
        }
    }

    /**
     * Test of getCredentials method with no credentials element specified. Should return null.
     */
    public void testGetCredentials_Unspecified() {
        assertNull("Returns an incorrect credentials element", server.getCredentials());
    }

    /**
     * Test of getCredentials method with credentials element specified. Should return the credentials element from the
     * createCredentials method.
     */
    public void testGetCredentials_Specified() {
        CredentialsElement credentials = server.createCredentials();

        assertSame("Returns an incorrect credentials element", credentials, server.getCredentials());
    }
}
