/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

/**
 * A simple implementation of the Server class for testing. It simply writes the server commands to the system output.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class SimpleServer extends Server {
    /**
     * The type of the server. It is initialized in the constructor and never changed.
     */
    private final String type;

    /**
     * Creates a new instance of SimpleServer.
     *
     * @param type the type of the server.
     */
    public SimpleServer(String type) {
        super(type);
        this.type = type;
    }

    /**
     * Returns the type of the server.
     *
     * @return the type of the server.
     */
    public String getType() {
        return type;
    }

    /**
     * Starts the server. This method should be called before running the tests.
     */
    public void startUp() {
        System.out.print(type + ":startUp();");
    }

    /**
     * Stops the server. This method should be called after running the tests.
     */
    public void shutDown() {
        System.out.print(type + ":shutDown();");
    }

    /**
     * Initializes the server with data. This method should be called before running the tests.
     */
    public void initData() {
        System.out.print(type + ":initData();");
    }

    /**
     * Cleans up the data for the server initialized by the initData() method. This method should be called after
     * running the tests.
     */
    public void cleanUpData() {
        System.out.print(type + ":cleanUpData();");
    }
}
