/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import com.topcoder.testframework.AntTaskInfo;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.integration.ant.container.AbstractContainer;
import org.apache.cactus.integration.ant.container.Container;
import org.apache.cactus.integration.ant.container.ContainerRunner;
import org.apache.cactus.integration.ant.container.jboss.JBoss3xContainer;
import org.apache.cactus.integration.ant.container.orion.Orion1xContainer;
import org.apache.cactus.integration.ant.container.orion.Orion2xContainer;
import org.apache.cactus.integration.ant.container.resin.Resin2xContainer;
import org.apache.cactus.integration.ant.container.resin.Resin3xContainer;
import org.apache.cactus.integration.ant.container.tomcat.Tomcat3xContainer;
import org.apache.cactus.integration.ant.container.tomcat.Tomcat4xContainer;
import org.apache.cactus.integration.ant.container.tomcat.Tomcat5xContainer;
import org.apache.cactus.integration.ant.container.weblogic.WebLogic7xContainer;
import org.apache.cactus.integration.ant.deployment.DeployableFile;
import org.apache.cactus.integration.ant.deployment.EarDeployableFile;
import org.apache.cactus.integration.ant.deployment.WarDeployableFile;
import org.apache.cactus.integration.ant.util.AntTaskFactory;
import org.apache.cactus.integration.ant.util.DefaultAntTaskFactory;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.BuildFileTest;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;

import java.io.File;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import java.net.URL;


/**
 * Unit tests for DefaultWebApplicationServer implementation. The tests consist of two parts: one directly against the
 * DefaultWebApplicationServer instance, the other indirectly through a sample build file for testing the startUp()
 * and shutDown() methods.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DefaultWebApplicationServerTest extends BuildFileTest {
    /**
     * The build file used to test for DefaultWebApplicationServer.
     */
    private static final String BUILD_FILE = "test_files/ServerAppTestTask.xml";

    /**
     * The DefaultWebApplicationServer instance to test against.
     */
    private DefaultWebApplicationServer server = null;

    /**
     * The value of the "dir" attribute.
     */
    private File dir = null;

    /**
     * The value of the "port" attribute.
     */
    private int port = -1;

    /**
     * The value of the "server" attribute.
     */
    private String name = null;

    /**
     * The value of the "warfile" attribute.
     */
    private File warFile = null;

    /**
     * The value of the "earfile" attribute.
     */
    private File earFile = null;

    /**
     * The value of the "timeout" attribute.
     */
    private long timeout = -1;

    /**
     * The value of the "testurl" attribute.
     */
    private URL testURL = null;

    /**
     * Creates a test of the specified name. Configures the sample build file to run the targets from.
     *
     * @param name the name of the test.
     */
    public DefaultWebApplicationServerTest(String name) {
        super(name);
        configureProject(BUILD_FILE);
    }

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(DefaultWebApplicationServerTest.class);

        return suite;
    }

    /**
     * Prepares a DefaultWebApplicationServer instance for testing. Also prepares the attributes supported by the
     * server.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void setUp() throws Exception {
        // prepare the attributes
        dir = new File("");
        port = 100;
        name = "10.20.30.40";
        warFile = new File("test_files/test.war");
        earFile = new File("test_files/test.ear");
        timeout = 10000;
        testURL = new URL("http://localhost/test/");

        // prepare the server
        AntTaskInfo taskInfo = new AntTaskInfo(new Project(), "taskName", new Location("build.xml"), new Target());

        server = new DefaultWebApplicationServer("tomcat4x");
        server.setAntTaskInfo(taskInfo);

        // set the attributes on the server
        server.setDir(dir);
        server.setPort(port);
        server.setServer(name);
        server.setWarFile(warFile);
    }

    /**
     * Tests the constructor with null type. Expects IllegalArgumentException.
     */
    public void testConstructor_NullType() {
        try {
            new DefaultWebApplicationServer(null);
            fail("Creates DefaultWebApplicationServer with null type");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with empty type. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyType() {
        try {
            new DefaultWebApplicationServer("");
            fail("Creates DefaultWebApplicationServer with empty type");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Tests the constructor with defined types. These include jboss3x, orion1x, orion2x, resin2x, resin3x, tomcat3x,
     * tomcat4x, tomcat5x, weblogic7x. Verifies if the container attribute is set to JBoss3xContainer,
     * Orion1xContainer, Orion2xContainer, Resin2xContainer, Resin3xContainer, Tomcat3xContainer, Tomcat4xContainer,
     * Tomcat5xContainer, WebLogic7xContainer.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_DefinedTypes() throws Exception {
        String[] types = {
                "jboss3x", "orion1x", "orion2x", "resin2x", "resin3x", "tomcat3x", "tomcat4x", "tomcat5x", "weblogic7x"
            };
        Class[] classes = {
                JBoss3xContainer.class, Orion1xContainer.class, Orion2xContainer.class, Resin2xContainer.class,
                Resin3xContainer.class, Tomcat3xContainer.class, Tomcat4xContainer.class, Tomcat5xContainer.class,
                WebLogic7xContainer.class
            };

        for (int i = 0; i < types.length; i++) {
            server = new DefaultWebApplicationServer(types[i]);
            assertEquals("Fails to set container attribute", classes[i], getField(server, "container").getClass());
        }
    }

    /**
     * Tests the constructor with undefined type. Expects BuildException.
     */
    public void testConstructor_UndefinedType() {
        try {
            new DefaultWebApplicationServer("undefined");
            fail("Creates DefaultWebApplicationServer with undefined type");
        } catch (BuildException e) {
            // good
        }
    }

    /**
     * Test of startUp method on ant task factory attribute. Verifies if the attribute is set in the container. The
     * attribute should be of type DefaultAntTaskFactory.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnAntTaskFactory() throws Exception {
        startUp();

        Container container = (Container) getField(server, "container");
        AntTaskFactory factory = (AntTaskFactory) getField(AbstractContainer.class, container, "antTaskFactory");

        assertNotNull("Fails to set ant task factory attribute", factory);
        assertTrue("Fails to set ant task factory attribute", factory instanceof DefaultAntTaskFactory);
    }

    /**
     * Test of startUp method with no warfile or earfile attribute specified. Verifies if the deployableFile attribute
     * is not set in the container.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnUnspecifiedDeployableFile()
        throws Exception {
        server.setWarFile(null);
        server.setTestURL(testURL);

        startUp();

        Container container = (Container) getField(server, "container");
        DeployableFile deployableFile = (DeployableFile) getField(AbstractContainer.class, container, "deployableFile");

        assertNull("Fails to set deployableFile attribute", deployableFile);
    }

    /**
     * Test of startUp method on warfile attribute. Verifies if the deployableFile attribute is set in the container.
     * The attribute should be of type WarDeployableFile.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnWarDeployableFile() throws Exception {
        server.setWarFile(warFile);

        startUp();

        Container container = (Container) getField(server, "container");
        DeployableFile deployableFile = (DeployableFile) getField(AbstractContainer.class, container, "deployableFile");

        assertNotNull("Fails to set deployableFile attribute", deployableFile);
        assertTrue("Fails to set deployableFile attribute", deployableFile instanceof WarDeployableFile);
    }

    /**
     * Test of startUp method on earfile attribute. Verifies if the deployableFile attribute is set in the container.
     * The attribute should be of type EarDeployableFile.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnEarDeployableFile() throws Exception {
        server.setWarFile(null);
        server.setEarFile(earFile);

        startUp();

        Container container = (Container) getField(server, "container");
        DeployableFile deployableFile = (DeployableFile) getField(AbstractContainer.class, container, "deployableFile");

        assertNotNull("Fails to set deployableFile attribute", deployableFile);
        assertTrue("Fails to set deployableFile attribute", deployableFile instanceof EarDeployableFile);
    }

    /**
     * Test of startUp method on dir attribute. Verifies if the attribute is set in the container.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnDir() throws Exception {
        startUp();

        Container container = (Container) getField(server, "container");

        assertEquals("Fails to set dir attribute", dir, invoke(container, "getDir"));
    }

    /**
     * Test of startUp method on server attribute. Verifies if the attribute is set in the container.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnServer() throws Exception {
        startUp();

        Container container = (Container) getField(server, "container");

        assertEquals("Fails to set server attribute", name, invoke(container, "getServer"));
    }

    /**
     * Test of startUp method on port attribute. Verifies if the attribute is set in the container.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnPort() throws Exception {
        startUp();

        Container container = (Container) getField(server, "container");

        assertEquals("Fails to set port attribute", new Integer(port), invoke(container, "getPort"));
    }

    /**
     * Test of startUp method on timeout attribute. Verifies if the attribute is set in the runner.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnTimeout() throws Exception {
        server.setTimeout(timeout);

        startUp();

        ContainerRunner runner = (ContainerRunner) getField(server, "runner");

        assertEquals("Fails to set timeout attribute", new Long(timeout), getField(runner, "timeout"));
    }

    /**
     * Test of startUp method with testURL attribute specified. Verifies if the testURL attribute is set in the runner
     * to the testURL itself. Also verifies if the "cactus.contextURL" system property is set to the testURL.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnSpecifiedTestURL() throws Exception {
        server.setTestURL(testURL);

        startUp();

        ContainerRunner runner = (ContainerRunner) getField(server, "runner");

        assertEquals("Fails to set testURL attribute", testURL, getField(runner, "testURL"));
        assertEquals("Fails to set \"cactus.contextURL\" system property", testURL.toString(),
            System.getProperty("cactus.contextURL"));
    }

    /**
     * Test of startUp method with no testURL attribute specified. Verifies if the testURL attribute is set in the
     * runner to the one composed by the deployable file, prepending with cactus query string. Also verifies if the
     * "cactus.contextURL" system property is set to the one composed by the deployable file.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testStartUp_OnUnspecifiedTestURL() throws Exception {
        startUp();

        ContainerRunner runner = (ContainerRunner) getField(server, "runner");

        assertEquals("Fails to set testURL attribute",
            "http://" + name + ":" + port + "/test/ServletRedirector?Cactus_Service=RUN_TEST",
            getField(runner, "testURL").toString());
        assertEquals("Fails to set \"cactus.contextURL\" system property", "http://" + name + ":" + port + "/test",
            System.getProperty("cactus.contextURL"));
    }

    /**
     * Test of startUp method with no dir attribute specified. Expects BuildException.
     */
    public void testStartUp_UnspecifiedDir() {
        expectBuildException("test_defaultwebserver_no_dir", "The dir attribute must be specified");
    }

    /**
     * Test of startUp method with no warfile and earfile attributes specified (and the testURL attribute is absent).
     * Expects BuildException.
     */
    public void testStartUp_UnspecifiedWarEar() {
        expectBuildException("test_defaultwebserver_no_war_ear",
            "The testurl attribute must be specified when both warfile and earfile attributes are not specified");
    }

    /**
     * Test of startUp method with both warfile and earfile attributes specified. Expects BuildException.
     */
    public void testStartUp_SpecifiedWarEar() {
        expectBuildException("test_defaultwebserver_both_war_ear",
            "The warfile and earfile attributes must not be specified together");
    }

    /**
     * Test of startUp method on container runner. Verifies if the startup message is logged. This also proves that the
     * server was started up successfully.
     */
    public void testStartUp_OnRunner() {
        executeTarget("test_defaultwebserver_runner");
        assertTrue("Fails to start the server", getLog().indexOf("Starting background thread") >= 0);
    }

    /**
     * Test of shutDown method on container runner. Verifies if the shutdown message is logged. This also proves that
     * the server was stopped successfully.
     */
    public void testShutDown_OnRunner() {
        executeTarget("test_defaultwebserver_runner");
        assertTrue("Fails to stop the server", getLog().indexOf("Stopping background thread") >= 0);
    }

    /**
     * Test of initData method. Simply tests the accessibility here since the method does nothing.
     */
    public void testInitData() {
        server.initData();
    }

    /**
     * Test of cleanUpData method. Simply tests the accessibility here since the method does nothing.
     */
    public void testCleanUpData() {
        server.cleanUpData();
    }

    /**
     * Test of setTimeout method with negative timeout. Expects IllegalArgumentException.
     */
    public void testSetTimeout_NegativeTimeout() {
        try {
            server.setTimeout(-1);
            fail("Sets negative timeout");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getTimeout method with no timeout specified. Should return 180000.
     */
    public void testGetTimeout_Unspecified() {
        assertEquals("Returns an incorrect timeout", 180000, server.getTimeout());
    }

    /**
     * Test of getTimeout and setTimeout methods with timeout specified. Verifies it by getting the timeout back after
     * setting it.
     */
    public void testGetSetTimeout_Specified() {
        long timeout;

        // try zero timeout
        timeout = 0;
        server.setTimeout(timeout);
        assertEquals("Returns an incorrect timeout", timeout, server.getTimeout());

        // try positive timeout
        timeout = 10;
        server.setTimeout(timeout);
        assertEquals("Returns an incorrect timeout", timeout, server.getTimeout());
    }

    /**
     * Test of getTestURL and setTestURL methods with test url specified. Verifies it by getting the test url back
     * after setting it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetSetTestURL_Specified() throws Exception {
        URL testURL = new URL("http://localhost/");

        server.setTestURL(testURL);
        assertEquals("Returns an incorrect test url", testURL, server.getTestURL());
    }

    /**
     * Test of getTestURL method with no test url specified. Should return null.
     */
    public void testGetTestURL_Unspecified() {
        assertNull("Returns an incorrect test url", server.getTestURL());
    }

    /**
     * Gets the field value of the given instance. The field name is also given.
     *
     * @param instance the instance whose field value is retrieved.
     * @param field the field name.
     *
     * @return the field value of the given instance.
     *
     * @throws Exception if error occurs when retrieving the value.
     */
    private static Object getField(Object instance, String field)
        throws Exception {
        return getField(instance.getClass(), instance, field);
    }

    /**
     * Gets the field value of the given instance. The field name and the class containing the field is also given.
     *
     * @param type the class containing the field to be retrieved.
     * @param instance the instance whose field value is retrieved.
     * @param field the field name.
     *
     * @return the field value of the given instance.
     *
     * @throws Exception if error occurs when retrieving the value.
     */
    private static Object getField(Class type, Object instance, String field)
        throws Exception {
        Field info = type.getDeclaredField(field);
        Object value;

        try {
            info.setAccessible(true);
            value = info.get(instance);
        } finally {
            info.setAccessible(false);
        }
        return value;
    }

    /**
     * Invokes the getter method of the specified name on the given instance. No method argument is given. For
     * primitive types, the returned object is subject to method invocation conversions.
     *
     * @param instance the instance whose method is invoked.
     * @param methodName the name of the method to invoke.
     *
     * @return the result of dispatching the method.
     *
     * @throws Exception if error occurs when invoking the method.
     */
    private static Object invoke(Object instance, String methodName)
        throws Exception {
        // locate the method
        Method method = instance.getClass().getMethod(methodName, null);

        // invoke the method, return the result
        return method.invoke(instance, null);
    }

    /**
     * Calls the startUp() method of the server, ignoring any exception during startup. This is used for testing if the
     * attributes are set in the container and runner.
     */
    private void startUp() {
        try {
            server.startUp();
        } catch (BuildException e) {
            // ignore any exception during startup
        }
    }
}
